import { Component, ElementRef, ViewEncapsulation, Input, Output, ViewChild, EventEmitter, ChangeDetectorRef } from '@angular/core';
/**
 * Implements a markdown component. Important: in order that the checkbox
 * feature is working, the rendered content set using the attribute needs
 * to be wrapped into a SafeValue instance. This prevents the sanitizer
 * to strip the input fields needed for the checkboxes from the rendered
 * html.
 */
var MarkdownComponent = /** @class */ (function () {
    function MarkdownComponent(cdr, elementRef) {
        this.cdr = cdr;
        this.elementRef = elementRef;
        this.fieldName = 'Description';
        this.inpRenderedText = '';
        this.inpRawText = '';
        this.rendering = false;
        this.saving = false;
        this.placeholder = 'This is place holder';
        this.editAllow = true;
        this.renderedHeight = 300;
        this.allowEmptySave = true;
        this.onActiveEditor = new EventEmitter();
        this.onSaveClick = new EventEmitter();
        this.showPreview = new EventEmitter();
        this.onCloseClick = new EventEmitter();
        this.enableShowMore = false;
        // these need to be public for the tests accessing them.
        this.renderedText = '';
        this.rawText = '';
        this.fieldEmpty = true;
        this.previousRawText = '';
        this.markdownViewExpanded = false;
        this.tabBarVisible = true;
        this.viewType = 'preview'; // markdown
        this.editorActive = false;
        this.showMore = false;
        this.inputsDisabled = false;
        this.previousRenderedText = '';
    }
    MarkdownComponent.prototype.ngOnChanges = function (changes) {
        if (changes.editAllow &&
            this.editAllow === false &&
            !changes.editAllow.isFirstChange) {
            this.closeClick();
        }
        if (Object.keys(changes).indexOf('inpRenderedText') > -1
            && typeof (this.inpRenderedText) === 'undefined') {
            console.warn('Markdown component change :: renderedText is passed undefined');
            this.renderedText = '';
        }
        else {
            this.renderedText = this.inpRenderedText;
        }
        if (Object.keys(changes).indexOf('inpRawText') > -1
            && typeof (this.inpRawText) === 'undefined') {
            console.warn('Markdown component change :: rawText is passed undefined');
            this.rawText = '';
        }
        else {
            this.rawText = this.inpRawText;
        }
    };
    MarkdownComponent.prototype.ngOnInit = function () {
        if (typeof (this.renderedText) === 'undefined') {
            console.warn('Markdown component init :: renderedText is passed undefined');
            this.renderedText = '';
        }
        if (typeof (this.rawText) === 'undefined') {
            console.warn('Markdown component init :: rawText is passed undefined');
            this.rawText = '';
        }
    };
    MarkdownComponent.prototype.ngAfterViewChecked = function () {
        if (this.editorBox) {
            this.boxHeight = this.editorBox.nativeElement.offsetHeight;
            if (this.boxHeight > this.renderedHeight) {
                this.enableShowMore = true;
            }
            else {
                this.enableShowMore = false;
            }
            this.cdr.detectChanges();
        }
        this.checkInputsDisabled();
    };
    MarkdownComponent.prototype.onInputEvent = function (event) {
        console.log('In-Markup Markdown input Event detected for input type: ' + event.type +
            ' with extraData ' + JSON.stringify(event.extraData));
        // we only support this interaction on checkboxes for now.
        // the mechanic is generic, add other controls below. In case,
        // you need to also add support for them in github-link-area as well.
        if (event.type === 'checkbox') {
            // disable the inputs so we don't get into async issues when
            // storing to the upstream component. This has to be done on
            // new supported input types as well.
            this.inputsDisabled = true;
            // process the checkbox clicked, find the markdown markup reference, update it.
            var activatedCheckboxIndex = event.extraData.checkboxIndex;
            var checked = event.extraData.checked;
            var markdownMarkup = this.rawText;
            // the JavaScript RegExp flavour makes it hard to to nth occurence
            // expressions, so we're doing it by hand here.
            var regex = /^ *[-*] *\[[ xX]*\]/gm;
            var m = void 0;
            var matchIndex = 0;
            // tslint:disable-next-line:no-conditional-assignment
            while ((m = regex.exec(markdownMarkup)) !== null) {
                // This is necessary to avoid infinite loops with zero-width matches.
                if (m.index === regex.lastIndex) {
                    regex.lastIndex++;
                }
                if (matchIndex === activatedCheckboxIndex && m.length > 0) {
                    // JavaScript does not have a replace by index method.
                    var matchLen = m[0].length;
                    var matchEndIndex = regex.lastIndex;
                    var matchStartIndex = matchEndIndex - matchLen;
                    var replaceStr = void 0;
                    if (checked)
                        replaceStr = m[0].replace(/\[[ ]*\]/, '[x]');
                    else
                        replaceStr = m[0].replace(/\[[xX]+\]/, '[ ]');
                    markdownMarkup =
                        markdownMarkup.substring(0, matchStartIndex) +
                            replaceStr +
                            markdownMarkup.substring(matchEndIndex, markdownMarkup.length);
                }
                matchIndex++;
            }
            this.rawText = markdownMarkup;
            // signal that the markdown has changed to the outside world.
            this.saveClick();
        }
        else {
            console.log('Input type ' + event.type + ' is not supported yet.');
        }
    };
    MarkdownComponent.prototype.onClickMarkdownTab = function () {
        if (this.viewType === 'preview') {
            this.viewType = 'markdown';
        }
    };
    MarkdownComponent.prototype.onClickPreviewTab = function () {
        var _this = this;
        if (this.viewType === 'markdown') {
            if (this.rawText !== this.editorInput.nativeElement.innerText.trim()) {
                // Emit raw text to get preview
                this.rawText = this.editorInput.nativeElement.innerText.trim();
                this.showPreview.emit({
                    rawText: this.rawText,
                    callBack: function (t, m) { return _this.renderPreview(t, m); }
                });
                this.rendering = true;
            }
            else {
                this.viewType = 'preview';
            }
        }
    };
    MarkdownComponent.prototype.checkInputsDisabled = function () {
        this.setPreviewInputsDisabled('input.markdown-checkbox', this.inputsDisabled);
    };
    // disables/enables the inputs on the rendered markup
    MarkdownComponent.prototype.setPreviewInputsDisabled = function (query, disabled) {
        var el = this.previewArea;
        if (el) {
            var queryElems = el.nativeElement.querySelectorAll(query);
            if (queryElems && queryElems.length > 0) {
                // we need to use a classic loop instead of forEach here
                // as forEach on NodeLists is not supported on every browser.
                // Example: Chrome works, but Protractor not.
                for (var i = 0; i < queryElems.length; ++i) {
                    if (disabled) {
                        queryElems[i].setAttribute('disabled', disabled ? 'true' : 'false');
                    }
                    else {
                        queryElems[i].removeAttribute('disabled');
                    }
                }
            }
        }
    };
    MarkdownComponent.prototype.enableEditor = function () {
        if (this.rawText === '') {
            this.activeEditor();
        }
        this.inputsDisabled = true;
    };
    MarkdownComponent.prototype.activeEditor = function () {
        if (this.editAllow) {
            // Activate the editor
            this.editorActive = true;
            // Show the markdown default view
            this.viewType = 'markdown';
            // Save current values
            this.previousRawText = this.rawText;
            this.previousRenderedText = this.renderedText;
            this.onActiveEditor.emit(true);
        }
    };
    MarkdownComponent.prototype.deactivateEditor = function () {
        // Deactivate the editor
        this.editorActive = false;
        // deactivate inputs when in edit mode
        this.inputsDisabled = false;
        // Show the preview default view
        this.viewType = 'preview';
    };
    MarkdownComponent.prototype.saveClick = function () {
        var _this = this;
        if (this.allowEmptySave || (!this.fieldEmpty && !this.allowEmptySave)) {
            if (this.viewType === 'markdown' &&
                this.previousRawText !== this.editorInput.nativeElement.innerText.trim()) {
                this.saving = true;
                this.onSaveClick.emit({
                    rawText: this.editorInput.nativeElement.innerText.trim(),
                    callBack: function (t, m) { return _this.saveUpdate(t, m); }
                });
            }
            else if (this.viewType === 'preview' &&
                this.previousRawText !== this.rawText) {
                this.saving = true;
                this.onSaveClick.emit({
                    rawText: this.rawText,
                    callBack: function (t, m) { return _this.saveUpdate(t, m); }
                });
            }
            else {
                this.deactivateEditor();
            }
        }
    };
    MarkdownComponent.prototype.editorKeyUp = function (event) {
        // Do not use this.event.srcElement. The "event" object is not consistent
        // across browsers. Chrome has event.srcElement while firefox has event.originalTarget.
        this.fieldEmpty = this.editorInput.nativeElement.innerText.trim() === '';
    };
    MarkdownComponent.prototype.closeClick = function () {
        // Restore saved previous data
        this.rawText = this.previousRawText;
        this.renderedText = this.previousRenderedText;
        this.deactivateEditor();
        this.onCloseClick.emit();
    };
    MarkdownComponent.prototype.renderPreview = function (rawText, renderedText) {
        this.rendering = false;
        if (typeof (rawText) === 'undefined') {
            console.warn('Markdown component preview callback :: rawText is passed undefined');
            this.rawText = '';
        }
        else {
            this.rawText = rawText;
        }
        if (typeof (renderedText) === 'undefined') {
            console.warn('Markdown component preview callback :: renderedText is passed undefined');
            this.renderedText = '';
        }
        else {
            this.renderedText = renderedText;
        }
        this.viewType = 'preview';
    };
    MarkdownComponent.prototype.saveUpdate = function (rawText, renderedText) {
        this.saving = false;
        if (typeof (rawText) === 'undefined') {
            console.warn('Markdown component save callback :: rawText is passed undefined');
            this.rawText = '';
        }
        else {
            this.rawText = rawText;
        }
        if (typeof (renderedText) === 'undefined') {
            console.warn('Markdown component save callback :: renderedText is passed undefined');
            this.renderedText = '';
        }
        else {
            this.renderedText = renderedText;
        }
        // finally, deactivate the editor.
        this.deactivateEditor();
    };
    MarkdownComponent.prototype.lengthToggle = function (showMore) {
        this.showMore = showMore;
    };
    MarkdownComponent.decorators = [
        { type: Component, args: [{
                    encapsulation: ViewEncapsulation.None,
                    selector: 'f8-markdown',
                    styles: [require('./markdown.component.css').toString()],
                    template: require('./markdown.component.html')
                },] },
    ];
    /** @nocollapse */
    MarkdownComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef, },
        { type: ElementRef, },
    ]; };
    MarkdownComponent.propDecorators = {
        'fieldName': [{ type: Input },],
        'inpRenderedText': [{ type: Input, args: ['renderedText',] },],
        'inpRawText': [{ type: Input, args: ['rawText',] },],
        'rendering': [{ type: Input },],
        'saving': [{ type: Input },],
        'placeholder': [{ type: Input },],
        'editAllow': [{ type: Input },],
        'renderedHeight': [{ type: Input },],
        'allowEmptySave': [{ type: Input },],
        'onActiveEditor': [{ type: Output },],
        'onSaveClick': [{ type: Output },],
        'showPreview': [{ type: Output },],
        'onCloseClick': [{ type: Output },],
        'editorInput': [{ type: ViewChild, args: ['editorInput',] },],
        'editorBox': [{ type: ViewChild, args: ['editorBox',] },],
        'previewArea': [{ type: ViewChild, args: ['previewArea',] },],
    };
    return MarkdownComponent;
}());
export { MarkdownComponent };
//# sourceMappingURL=markdown.component.js.map